// partition:  a container for organizing a set of sequential integers
//             into non-overlapping groups.
// Version:    %VERSION%
// Copyright:  2013 Matthew T. Busche.
//
// This program is free software:  you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.

package org.mattbusche.util.partition.examples;

// Static imports of enums to simplify syntax:
//
import static org.mattbusche.util.partition.examples.Crayon.*;
import static org.mattbusche.util.partition.examples.CrayonState.User.*;
import static org.mattbusche.util.partition.examples.CrayonState.Quality.*;

/** This sample program shows how to use a partition to track the states of a
 ** box of 8 crayons being shared by two children, Becky and Cate, to draw
 ** pictures.
 **/
public class Crayons {

    public static void main(String[] args) {
        try {
            CrayonManager manager = new CrayonManager();

            Crayon c;

            // Becky grabs three crayons: preferrably orange, blue, and a sharp one.
            // She dulls the first two.
            //
            c = manager.findPreferred(ORANGE);
            manager.setState(c, BECKY, DULL);

            c = manager.findPreferred(BLUE);
            manager.setState(c, BECKY, DULL);

            c = manager.findPreferred();
            manager.setUser(c, BECKY);

            // Cate grabs two crayons, preferrably blue and green.
            // She dulls the first one.
            //
            c = manager.findPreferred(BLUE);
            manager.setState(c, CATE, DULL);

            c = manager.findPreferred(GREEN);
            manager.setUser(c, CATE);

            // Becky returns all her crayons to the box...
            //
            manager.moveAll(BECKY, BOX);

            // ...and then grabs a sharp one...
            //
            c = manager.find(SHARP);
            manager.setUser(c, BECKY);

            // ...and makes it dull...
            //
            manager.setQuality(c, DULL);

            // Cate returns her dullest crayon to the box.
            //
            c = manager.findPreferred(DULL, CATE);
            manager.setUser(c, BOX);

            // Cate notices the sharp ones are disappearing fast,
            // so she grabs all the remaining sharp ones...
            //
            manager.moveAll(BOX, CATE, SHARP);

            // Becky gets mad and shows dad.
            //
            System.out.println("\nFinal States:");
            System.out.println(manager);
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }
}
